import os
import boto3
from surprise import dump
from common_utils_pkg.Model import Model

class ModelInterface:
    def __init__(self, bucket_name='se4ai-group-4-aws', aws_key=None, aws_secret=None):
        self.s3_client = boto3.client('s3',aws_access_key_id=aws_key, aws_secret_access_key=aws_secret)
        self.bucket_name = bucket_name 

    def download_model(self, model_name):
        # TODO: we can introduce versioning to S3 here
        temp_model_path = 'tmp_model'
        s3_client.download_file(self.bucket_name, model_name, temp_model_path)
        predictions, algo = dump.load(file_name=temp_model_path)
        os.remove(temp_model_path)
        return Model(algo)

    def save_model(self, model_name, model):
        temp_model_path = 'tmp_model'
        dump.dump(file_name=temp_model_path, predictions=None, algo=model)
        s3_client.upload_file(temp_model_path, self.bucket_name, model_name)
        os.remove(temp_model_path)


import os
from sqlalchemy import create_engine, Column, Text, SmallInteger
from sqlalchemy.ext.declarative import declarative_base
from sqlalchemy.orm import sessionmaker

class Database:
    def __init__(self, user, password, host, port, database, **kwargs):
        self.engine = create_engine('postgresql://{}:{}@{}:{}/{}'.format(
                user, password, host, port, database
            ), **kwargs)
        Base = declarative_base(self.engine)
        self.Movie = self._create_movie_class(Base)
        self.User = self._create_user_class(Base)
        self.Rating = self._create_rating_class(Base)
        self.Recommendation = self._create_recommendation_class(Base)

    def load_session(self):
        # TODO: should we checkout scoped sessions
        Session = sessionmaker(bind=self.engine)
        return Session()
 
    def _create_movie_class(self, base):
        class Movie(base):
            __tablename__ = 'movies'
            __table_args__ = ({'autoload': True})
        return Movie
    
    def _create_user_class(self, base):
        class User(base):
            __tablename__ = 'users'
            __table_args__ = {'autoload':True}
        return User

    def _create_rating_class(self, base):
        class Rating(base):
            __tablename__ = 'ratings'

            movie_id = Column(Text, primary_key =True)
            user_id = Column(Text, primary_key =True)
            rating = Column(SmallInteger)
        return Rating

    def _create_recommendation_class(self, base):
        class Recommendation(base):
            __tablename__ = 'recommendations'
            __table_args__ = ({'autoload': True})
        return Recommendation
    



class Model:
    def __init__(self, model):
        self.model = model

    def predict(self, user_id, user_movie_list= [], top_n=20, ):
        """The user_movie_list is the list of movies the user has not seen"""

        movie_rating_predictions = []
        for movie in user_movie_list:
            predicted_rating = self.model.predictions(user_id, movie)
            movie_rating_predictions.append((movie, predicted_rating))

        movie_ranking = sorted(movie_rating_predictions, key = lambda movie_rating_tuple: movie_rating_tuple[1], reverse=True)
        return [movie_rating_tuple[0] for movie_rating_tuple in movie_ranking[:20]]




import setuptools

with open("README.md", "r") as fh:
    long_description = fh.read()

setuptools.setup(
    name="se4ai-group4-common-utils", # Replace with your own username
    version="0.1.0",
    author="Daniel Biales",
    author_email="dbiales@andrew.cmu.edu",
    description="A package for common utility functions that our project needs",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/cmu-seai/group-project-se4ai-group-4",
    packages=setuptools.find_packages(),
    classifiers=[
        "Programming Language :: Python :: 3",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ],
    python_requires='>=3.6',
    install_requires=['wheel','boto3','psycopg2-binary','sqlalchemy','surprise']
)
